@@ -71,24 +71,6 @@ ProcInfo* ProcInfo::find(pid_t pid)
 }
 
 
-
-static void
-set_proctree_reorderable(ProcData *procdata)
-{
-    GList *columns, *col;
-    GtkTreeView *proctree;
-
-    proctree = GTK_TREE_VIEW(procdata->tree);
-
-    columns = gtk_tree_view_get_columns (proctree);
-
-    for(col = columns; col; col = col->next)
-        gtk_tree_view_column_set_reorderable(static_cast<GtkTreeViewColumn*>(col->data), TRUE);
-
-    g_list_free(columns);
-}
-
-
 static void
 cb_columns_changed(GtkTreeView *treeview, gpointer user_data)
 {
@@ -146,8 +128,6 @@ proctable_set_columns_order(GtkTreeView *treeview, GSList *order)
 
         id = GPOINTER_TO_INT(it->data);
 
-        g_assert(id >= 0 && id < NUM_COLUMNS);
-
         cur = my_gtk_tree_view_get_column_with_sort_column_id(treeview, id);
 
         if(cur && cur != last)
@@ -159,7 +139,6 @@ proctable_set_columns_order(GtkTreeView *treeview, GSList *order)
 }
 
 
-
 GSList*
 proctable_get_columns_order(GtkTreeView *treeview)
 {
@@ -183,27 +162,42 @@ proctable_get_columns_order(GtkTreeView *treeview)
     return order;
 }
 
-void
-cb_proctable_column_resized(GtkWidget *widget)
+static guint timeout_id = 0;
+static GtkTreeViewColumn *current_column;
+
+static gboolean
+save_column_width (gpointer data)
 {
-    GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN(widget);
     gint width;
     gchar *key;
     int id;
     GSettings *settings;
-    gint saved_width;
 
-    settings = g_settings_get_child (ProcData::get_instance()->settings, "proctree");
-    id = gtk_tree_view_column_get_sort_column_id (column);
-    width = gtk_tree_view_column_get_width (column);
+    settings = g_settings_get_child (G_SETTINGS (data), "proctree");
+    id = gtk_tree_view_column_get_sort_column_id (current_column);
+    width = gtk_tree_view_column_get_width (current_column);
+
     key = g_strdup_printf ("col-%d-width", id);
-
-    g_settings_get (settings, key, "i", &saved_width);
-    if (saved_width!=width)
-    {
-        g_settings_set_int(settings, key, width);
-    }
+    g_settings_set_int(settings, key, width);
     g_free (key);
+
+    if (timeout_id) {
+        g_source_remove (timeout_id);
+        timeout_id = 0;
+    }
+
+    return FALSE;
+}
+
+static void
+cb_proctable_column_resized(GtkWidget *widget, GParamSpec *pspec, gpointer data)
+{
+    current_column = GTK_TREE_VIEW_COLUMN(widget);
+
+    if (timeout_id)
+        g_source_remove (timeout_id);
+
+    timeout_id = g_timeout_add (250, save_column_width, data);
 }
 
 static gboolean
@@ -319,7 +313,6 @@ proctable_new (ProcData * const procdata)
                                          search_equal_func,
                                          NULL,
                                          NULL);
-    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (proctree), TRUE);
     g_object_unref (G_OBJECT (model));
 
     selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (proctree));
@@ -342,8 +335,9 @@ proctable_new (ProcData * const procdata)
     gtk_tree_view_column_set_sort_column_id (column, COL_NAME);
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
-    gtk_tree_view_column_set_min_width (column, 1);
-    g_signal_connect(G_OBJECT(column), "notify::width", G_CALLBACK(cb_proctable_column_resized), NULL);
+    gtk_tree_view_column_set_min_width (column, 20);
+    gtk_tree_view_column_set_reorderable (column, TRUE);
+    g_signal_connect(G_OBJECT(column), "notify::fixed-width", G_CALLBACK(cb_proctable_column_resized), procdata->settings);
     gtk_tree_view_append_column (GTK_TREE_VIEW (proctree), column);
     gtk_tree_view_set_expander_column (GTK_TREE_VIEW (proctree), column);
 
@@ -357,9 +351,10 @@ proctable_new (ProcData * const procdata)
         col = gtk_tree_view_column_new();
         gtk_tree_view_column_pack_start(col, cell, TRUE);
         gtk_tree_view_column_set_title(col, _(titles[i]));
-        gtk_tree_view_column_set_resizable(col, TRUE);
         gtk_tree_view_column_set_sort_column_id(col, i);
-        g_signal_connect(G_OBJECT(col), "notify::width", G_CALLBACK(cb_proctable_column_resized), NULL);
+        gtk_tree_view_column_set_resizable(col, TRUE);
+        gtk_tree_view_column_set_sizing(col, GTK_TREE_VIEW_COLUMN_FIXED);
+        g_signal_connect(G_OBJECT(col), "notify::fixed-width", G_CALLBACK(cb_proctable_column_resized), procdata->settings);
         gtk_tree_view_column_set_reorderable(col, TRUE);
         gtk_tree_view_append_column(GTK_TREE_VIEW(proctree), col);
 
@@ -437,7 +432,7 @@ proctable_new (ProcData * const procdata)
                 break;
         }
 
-        // xaling
+        // xalign
         switch(i)
         {
             case COL_VMSIZE:
@@ -457,11 +452,10 @@ proctable_new (ProcData * const procdata)
         // sizing
         switch (i) {
             case COL_ARGS:
-                gtk_tree_view_column_set_sizing(col, GTK_TREE_VIEW_COLUMN_FIXED);
                 gtk_tree_view_column_set_min_width(col, 150);
                 break;
             default:
-                gtk_tree_view_column_set_min_width(column, 20);
+                gtk_tree_view_column_set_min_width(col, 20);
                 break;
         }
     }
@@ -470,8 +464,6 @@ proctable_new (ProcData * const procdata)
 
     procdata->tree = proctree;
 
-    set_proctree_reorderable(procdata);
-
     procman_get_tree_state (procdata->settings, proctree, "proctree");
 
     /* Override column settings by hiding this column if it's meaningless: */
@@ -807,7 +799,12 @@ update_info (ProcData *procdata, ProcInfo *info)
 
     info->set_user(procstate.uid);
 
-    info->pcpu = (proctime.rtime - info->cpu_time) * 100 / procdata->cpu_total_time;
+    // if the cpu time has increased reset the status to running
+    // regardless of kernel state (https://bugzilla.gnome.org/606579)
+    guint64 difference = proctime.rtime - info->cpu_time;
+    if (difference > 0)
+        info->status = GLIBTOP_PROCESS_RUNNING;
+    info->pcpu = difference * 100 / procdata->cpu_total_time;
     info->pcpu = MIN(info->pcpu, 100);
 
     if (not procdata->config.solaris_mode)
@@ -992,7 +989,7 @@ refresh_list (ProcData *procdata, const pid_t* pid_list, const guint n)
 }
 
 
-void
+static void
 proctable_update_list (ProcData * const procdata)
 {
     pid_t* pid_list;
@@ -1013,7 +1010,6 @@ proctable_update_list (ProcData * const procdata)
             if (procdata->config.show_tree)
             {
                 selection.save(procdata->tree);
-                proctable_clear_tree(procdata);
             }
             break;
 
@@ -1023,7 +1019,6 @@ proctable_update_list (ProcData * const procdata)
             if (procdata->config.show_tree)
             {
                 selection.save(procdata->tree);
-                proctable_clear_tree(procdata);
             }
             break;
     }
@@ -1047,7 +1042,7 @@ proctable_update_list (ProcData * const procdata)
 
 
 void
-proctable_update_all (ProcData * const procdata)
+proctable_update (ProcData * const procdata)
 {
     char* string;
 
